export default abstract class Optional<T> {
    readonly abstract isPresent: boolean;
    readonly isEmpty: boolean;
    abstract get(): T;
    abstract ifPresent(consumer: (value: T) => void): void;
    abstract ifPresentOrElse(consumer: (value: T) => void, emptyAction: () => void): void;
    abstract filter(predicate: (value: T) => boolean): Optional<T>;
    abstract map<U>(mapper: (value: T) => U): Optional<U>;
    abstract flatMap<U>(mapper: (value: T) => Optional<U>): Optional<U>;
    abstract or(supplier: () => Optional<T>): Optional<T>;
    abstract orElse(another: T): T;
    abstract orElseGet(supplier: () => T): T;
    abstract orElseThrow<U>(errorSupplier: () => U): T;
    static of<T>(value: T): Optional<T>;
    static ofNonNull<T>(value: T): Optional<T>;
    static ofNullable<T>(nullable: T | null | undefined): Optional<T>;
    static empty<T>(): Optional<T>;
}
